// -------------------------------------------------------------------------
// File:        LoadLanguage.cpp
//
// Purpose:     These functions are used to map and unmap functions in a
//              LanguageModule DLL into the address of the current process by
//              dynamically loading the DLL at runtime.
//
// Notes:       - This file only needs to be modified if a new function will
//                be exported from ALL board DLLs.
//   -------------------------------------------------------------------------
#include <stdio.h>
#include <winsock2.h>
#include <windows.h>

#include "..\Languages\LanguageModule.h"
#include "LoadLanguage.h"

TCreateLanguageModule myCreateLanguageModule;
TReleaseLanguageModule myReleaseLanguageModule;

// -------------------------------------------------------------------------
// Name:        LoadLanguageModuleFunctions
//
// Purpose:     Assigns the creation and destruction functions for use by LoadLanguageModule
//              and UnloadLanguageModule from the language module DLL specified
//              by handle languageLibrary.  If unsuccessful, neither LoadLanguageModule
//              or UnloadLanguageModule will be effective, and LoadLanguageModuleFunctions
//              will return false.
// -------------------------------------------------------------------------
bool LoadLanguageModuleFunctions(HINSTANCE languageLibrary)
{
#ifdef _MSC_VER
   const LPCSTR createLanguageModule = "CreateLanguageModule";
   const LPCSTR releaseLanguageModule = "ReleaseLanguageModule";
#else
   const LPCSTR createLanguageModule = "_CreateLanguageModule";
   const LPCSTR releaseLanguageModule = "_ReleaseLanguageModule";
#endif

   // Load creation function
   if((myCreateLanguageModule = (TCreateLanguageModule) GetProcAddress(languageLibrary, createLanguageModule)) == NULL)
   {
      return(false);
   }

   // Load destruction function
   if((myReleaseLanguageModule = (TReleaseLanguageModule) GetProcAddress(languageLibrary, releaseLanguageModule)) == NULL)
   {
      myCreateLanguageModule = NULL;
      return(false);
   }

   return(true);
}

// -------------------------------------------------------------------------
// Name:        LoadLanguageModule
//
// Purpose:     Loads a language module DLL, creates a Voice object, and updates
//              pInfo appropriately.
// -------------------------------------------------------------------------
bool LoadLanguageModule(PIVRINFO pInfo, const char* languageDLL)
{
   if(pInfo == NULL)
   {
      return(false);
   }

   if(pInfo->ivLog != NULL)
   {
      pInfo->ivLog->Print("      Loading language '%s'\n", languageDLL);
   }

   // Unload currently loaded language module DLL if it is different
   if((pInfo->languageLibrary != NULL) && (GetModuleHandle(languageDLL) != pInfo->languageLibrary))
   {
      UnloadLanguageModule(pInfo);
   }

   // Load language module DLL, or increment count if already loaded
   pInfo->languageLibrary = LoadLibrary(languageDLL);

   if(pInfo->languageLibrary == NULL)
   {
      if(pInfo->ivLog != NULL)
      {
         pInfo->ivLog->Print("      ERROR: Failed to load language module\n");
      }
      return(false);
   }

   // Load creation and destruction functions from language module
   if(!LoadLanguageModuleFunctions(pInfo->languageLibrary))
   {
      if(pInfo->ivLog != NULL)
      {
         pInfo->ivLog->Print("      ERROR: Failed to load language module functions\n");
      }
      return(false);
   }

   // Create new LanguageModule object for pInfo
   pInfo->language = myCreateLanguageModule();

   return(pInfo->language != NULL);
}

// -------------------------------------------------------------------------
// Name:        UnloadLanguageFunctions
//
// Purpose:     Destroys the LanguageModule object in pInfo, unloads the language
//              module DLL, and updates pInfo appropriately
// -------------------------------------------------------------------------
void UnloadLanguageModule(PIVRINFO pInfo)
{
   if(pInfo == NULL)
   {
      return;
   }

   // Destroy current LanguageModule object
   if((pInfo->language != NULL) && (myReleaseLanguageModule != NULL))
   {
      myReleaseLanguageModule(pInfo->language);
   }

   // Reset pointers
   pInfo->language = NULL;
   pInfo->languageLibrary = NULL;
   myCreateLanguageModule = NULL;
   myReleaseLanguageModule = NULL;
}

