#include "..\inet\SIPTransaction.h"
#include "StringSupport.h"
#include "..\boardDLLs\brd_generic.h"
#include "TFEngine_communication.h"
#include "step_SIPRegister.h"

SIPRegister::SIPRegister(STEPREC& stepRecord) : Step(stepRecord)
{
   registrar = stepRecord.s1.arr;
   URIToRegister = stepRecord.s2.arr;
   registrant = stepRecord.s3.arr;
   contact = stepRecord.s4.arr;
   expirationTime = stepRecord.s5.arr;
   responseVar = stepRecord.s6.arr;
   timeout = stepRecord.s7.arr;
}

SIPRegister::~SIPRegister(void)
{
}

const char* SIPRegister::GetStepName(void) const
{
   return("Register SIP URI");
}

IV_STATUS SIPRegister::Execute(PIVRINFO pInfo)
{
   IV_STATUS status;
   char* value;
   // Evaluate all parameters before exiting
   bool bParmsValid = true;
   int ret;
   ACE_INET_Addr host;
   char number[14];
   unsigned short registrarPort = 5060;
   string::size_type index;
   ACE_INET_Addr peer;
   LARGE_INTEGER liTimeout;
   HANDLE waitObjs[3];
   HANDLE hTimer;
   SIPTransaction sip(pInfo->ivLog);
   int iTimeout;
   unsigned int cseq = 1;
   string callID;
   ESCTime sysTime;
   string via;
   string parsedRegistrar;
   string parsedURIToRegister;
   string parsedRegistrant;
   string parsedContact;
   string parsedExpirationTime;
   string tempStr;

   if(Status(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FAIL);
   }

   if((value = pInfo->variables->ParseParameter("Registrar", registrar.c_str())) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      parsedRegistrar = value;
      delete[] value;

      // If application supplied "sip:", strip it out
      for(size_t i=0; i<4 && i<parsedRegistrar.length(); ++i)
      {
         tempStr += (char) tolower(parsedRegistrar[i]);
      }
      if(tempStr == "sip:")
      {
         parsedRegistrar = parsedRegistrar.substr((size_t) 4, string::npos);
      }
   }

   if((value = pInfo->variables->ParseParameter("URI to Register (To)", URIToRegister.c_str())) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      parsedURIToRegister = value;
      delete[] value;
   }

   if((value = pInfo->variables->ParseParameter("Registrant (From)", registrant.c_str())) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      parsedRegistrant = value;
      delete[] value;
   }

   if((value = pInfo->variables->ParseParameter("Contact (From)", contact.c_str(), "")) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      parsedContact = value;
      delete[] value;
   }

   if((value = pInfo->variables->ParseParameter("Expiration Time", expirationTime.c_str(),
      "", "Empty Expiration Time defaults to none")) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      parsedExpirationTime = value;
      delete[] value;
   }

   if((value = pInfo->variables->ParseParameter("TIMEOUT", timeout.c_str(), "@TIMEOUT")) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      iTimeout = atoi(value);
      delete[] value;

      if((iTimeout < 0) && (iTimeout != FOREVER))
      {
		   pInfo->ivLog->Print("      ERROR: TIMEOUT '%s' must be >= 0\n",
            pInfo->variables->IsParsedEmpty(timeout.c_str()) ? "@TIMEOUT" : timeout.c_str());
         bParmsValid = false;
      }
   }

   if(!bParmsValid)
   {
      return(IVR_FATAL);
   }

   sysTime.Update();

   callID = (string) sysTime.GetDate("") + (string) sysTime.GetTime("");
   sprintf(number, "%d", pInfo->iAppID);
   callID += (string) "-" + (string) number;
   callID += GenerateUniqueString(8, 'a', 'z');

   via = (string) sip.Request()->GetVersion() + (string) "/UDP ";
   sip.GetHostAddress(host);
   sprintf(number, "%d", host.get_port_number());
   via += (string) Socket::GetHostAddress() + (string) ":" + (string) number;
   via += (string) ";branch=z9hG4bk" + callID;

   callID += (string) "@" + (string) Socket::GetHostAddress();

   sip.Request()->SetMethod("REGISTER");
   sip.Request()->SetURI(((string) "sip:" + parsedRegistrar).c_str());
   sip.Request()->SetField("To", parsedURIToRegister.c_str());
   sip.Request()->SetField("From", (parsedRegistrant + (string) ";tag=" + GenerateUniqueString(4, 'a', 'z')).c_str());
   sprintf(number, "%d", cseq);
   sip.Request()->SetField("CSeq", ((string) number + (string) " REGISTER").c_str());
   sip.Request()->SetField("Call-ID", callID.c_str());
   sip.Request()->SetField("Max-Forwards", "70");
   sip.Request()->SetField("Via", via.c_str());
   if(parsedContact.length() > 0)
   {
      sip.Request()->SetField("Contact", parsedContact.c_str());
   }
   if(parsedExpirationTime.length() > 0)
   {
      sip.Request()->SetField("Expires", parsedExpirationTime.c_str());
   }

   if((index = parsedRegistrar.find(':')) != string::npos)
   {
      registrarPort = (unsigned short) atoi(parsedRegistrar.substr(index + 1).c_str());
      parsedRegistrar = parsedRegistrar.substr(0, index);
   }

   sprintf(number, "%d", registrarPort);

   if(peer.set(number, parsedRegistrar.c_str()) != 0)
   {
      pInfo->ivLog->Print("      ERROR: Failed to set registrar address\n");
      pInfo->ivLog->Print("         '%s'\n", Socket::WSAErrorString(errno));
      return(IVR_FAIL);
   }

   pInfo->ivLog->Print("      Sending request\n");

   if((ret = sip.Send(peer)) != Socket::SUCCESS)
   {
      pInfo->ivLog->Print("      ERROR: Failed to send registration\n");
      pInfo->ivLog->Print("         '%s'\n", Socket::WSAErrorString(ret));
      return(IVR_FAIL);
   }

   // Set timeout timer
   hTimer = CreateWaitableTimer(0, false, 0);
   if(iTimeout != FOREVER)
   {
      liTimeout.QuadPart = -(LONGLONG) iTimeout * 10000;
      SetWaitableTimer(hTimer, &liTimeout, 0, 0, 0, false);
   }

   // Set up events array
   waitObjs[0] = pInfo->messages->GetMessageSignal();  // system event such as shutdown
   waitObjs[1] = hTimer;                               // Silence timeout
   waitObjs[2] = sip.GetEventHandle();                 // signal from socket

   while(sip.Response()->GetStatusCode() < 200)
   {
      sip.Response()->ClearText();                     // clear previous response
      while(!sip.Response()->HaveCompleteMessage())
      {
         switch(WaitForMultipleObjects(3, waitObjs, false, INFINITE))
         {
            // System event
            case WAIT_OBJECT_0:
            {
               status = pInfo->ivBoard->ProcessSystemEvent();

               if(status != IVR_SUCCESS)
               {
                  return(status);
               }

               break;
            }

            // Connection timeout
            case WAIT_OBJECT_0 + 1:
            {
               pInfo->ivLog->Print("      ERROR: Timed out waiting for response\n");
               return(IVR_TIMEOUT);
            }

            // Socket signalled
            case WAIT_OBJECT_0 + 2:
            {
               if((ret = sip.Receive(peer)) != Socket::SUCCESS)
               {
                  pInfo->ivLog->Print("      Error receiving message\n");
                  pInfo->ivLog->Print("         '%s'\n", Socket::WSAErrorString(ret));

                  return(IVR_FAIL);
               }

               break;
            }

            // WaitForMultipleObjects failed
            case WAIT_FAILED:
            {
               DWORD errorID = GetLastError();
               char MsgBuf[1024];

               FormatMessage(
                  FORMAT_MESSAGE_FROM_SYSTEM,
                  0,
                  errorID,
                  MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT),
                  MsgBuf,
                  1024,
                  0
               );

               pInfo->ivLog->Print("      ERROR: WaitForMultipleObjects returned: %x: %s", errorID, MsgBuf);
               return(IVR_FAIL);
            }
         }
      }
      switch(sip.Response()->GetStatusCode())
      {
         case 100:
         {
            pInfo->ivLog->Print("         Response: 100 Trying\n");
            break;
         }
         case 200:
         {
            pInfo->ivLog->Print("         Response: 200 Registration Successful\n");
            break;
         }
         default:
         {
            pInfo->ivLog->Print("         Response:\n");
            pInfo->ivLog->Print("%s\n", sip.Response()->GetText());
         }
      }
   } // end while: sip.Response()->GetStatusCode() < 200

   if(!responseVar.empty() && !pInfo->variables->Set(responseVar.c_str(), sip.Response()->GetText()))
   {
      pInfo->ivLog->Print("      ERROR: Failed to set response variable '%s'\n", responseVar.c_str());
      return(IVR_FATAL);
   }

   if(sip.Response()->GetStatusCode() != 200)
   {
      pInfo->ivLog->Print("      ERROR: Failed to register\n");
      pInfo->ivLog->Print("         '%s'\n", sip.Response()->GetReasonPhrase());
      return(IVR_FAIL);
   }

   pInfo->ivLog->Print("      Registration successful\n");

   return(IVR_SUCCESS);
}
