#include <stdlib.h>             // Needed for "atoi".
#include "step_getdigit.h"
#include "..\BoardDLLs\brd_generic.h"
#include "..\Voice\Voice.h"

StepGetDigit::StepGetDigit(STEPREC& stepRecord) : StepTones(stepRecord)
{
   timeout = stepRecord.s2.arr;
   varNm = stepRecord.s1.arr;
   confidenceVar = stepRecord.s7.arr;
   thresholdConfidence = stepRecord.s6.arr;
   grammar = stepRecord.s4.arr;
   grammarText = stepRecord.s5.arr;
   handleVar = stepRecord.s8.arr;
   retainAudioOnThreshold = (bool) stepRecord.u1;
   rememberHandle = (bool) stepRecord.u2;
}

StepGetDigit::~StepGetDigit(void)
{
}

const char* StepGetDigit::GetStepName(void) const
{
	return("Menu");
}

IV_STATUS StepGetDigit::Execute (PIVRINFO pInfo)
{
   IV_STATUS  status;
   // Evaluate all parameters before exiting
   bool bParmsValid = true;
   int iTimeout;
   unsigned int endOfSpeechTimeout = pInfo->ivBoard->GetSREndOfSpeechTimeout();
   char iNext = '\0';
   char ttDig = '\0';      // used as a flag to indicate SR interrupted by a digit
   char keyPressed[2];
   char* value;
   string parsedGrammar;
   string parsedGrammarText;
   char concept[1024];
   unsigned int maxlength = 1023;
   unsigned int confidence;
   unsigned int iThresholdConfidence;
   bool nonMenuSRValue = false;
   Voice::SR_GRAMMAR grammarType = Voice::FILE; // assume FILE grammar unless we find out otherwise

	// String large enough for 0 to 100 %
   char confidenceString[4];
   concept[0] = '\0';

   if(Status(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FAIL);
   }

	if(PortRequired(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FATAL);
   }

   // Step properties relevant to both TT and SR
   // ------------------------------------------
   if((value = pInfo->variables->ParseParameter("TIMEOUT", timeout.c_str(), "@TIMEOUT")) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      iTimeout = atoi(value);
      delete[] value;

      if(iTimeout < 0)
      {
		   pInfo->ivLog->Print("      ERROR: TIMEOUT '%s' must be >= 0\n", timeout.c_str());
         bParmsValid = false;
      }
   }

   if((value = pInfo->variables->ParseParameter("Grammar", grammar.c_str(), "", "")) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      parsedGrammar = value;
      delete[] value;
   }

   if((value = pInfo->variables->ParseParameter("Grammar Text", grammarText.c_str(), "", "")) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      parsedGrammarText = value;
      delete[] value;
   }

   // TT only
   if((pInfo->speechRecognition == NULL) || (parsedGrammar.empty() && parsedGrammarText.empty()))
   {
      if(!bParmsValid)
      {
         return(IVR_FATAL);
      }

      if(((status = pInfo->ivBoard->GetDigit(iNext, iTimeout)) != IVR_SUCCESS) && (status != IVR_TIMEOUT))
      {
         return(status);
      }

      // Touch tones have confidence 100%, and no voice has confidence 0%
      confidence = status == IVR_SUCCESS ? 100 : 0;

      if(!confidenceVar.empty())
      {
         sprintf(confidenceString, "%d", confidence);

         // Set confidence variable with confidence
         // Touch tones get 100% confidence automatically
         if(!pInfo->variables->Set(confidenceVar.c_str(), confidenceString))
         {
            pInfo->ivLog->Print("      ERROR: Failed to set Confidence variable '%s' to '%s'\n", confidenceVar.c_str(), confidenceString);
            return(IVR_FATAL);
         }
      }

      // SR Result = SUCCESS or TIMEOUT
      // Capture SR result for SpeechTrainer
      if(pInfo->speechRecognition != NULL)
      {
         pInfo->speechRecognition->SRCaptureResult(iNext, "", confidence, status);
      }

      if(status != IVR_SUCCESS)
      {
         // timeout returns immediately
         return(status);
      }
   }
   // SR, with TT input allowed
   else
   {
      if(!thresholdConfidence.empty())
      {
         if((value = pInfo->variables->ParseParameter("Threshold confidence", thresholdConfidence.c_str())) == NULL)
         {
            bParmsValid = false;
         }
         else
         {
            iThresholdConfidence = atoi(value);
            delete[] value;
         }
      }
      else
      {
         pInfo->ivLog->Print("      No threshold confidence\n");
      }

      pInfo->ivLog->Print("      End of speech timeout: %d\n", endOfSpeechTimeout);

      if(parsedGrammar.empty())
      {
         // Base Grammar not supplied, Grammar Text will be used as base instead
         grammarType = Voice::TEXT;
      }

      if(!bParmsValid)
      {
         return(IVR_FATAL);
      }

      // Capture SR starting information for SpeechTrainer
      pInfo->speechRecognition->SRCaptureStartStats( pInfo->pszAppFile,    // SR step TAP
                                                 GetCurrentStep(),         // SR step number
                                                 GetStepName(),            // SR step name
                                                 iTimeout,                 // no voice t/o (int)
                                                 endOfSpeechTimeout,       // end speech t/o (unsigned int)
                                                 iThresholdConfidence,     // threshold (unsigned int)
                                                 retainAudioOnThreshold,   // retainAudio (bool)
                                                 grammarType,              // grammar type either FILE or TEXT
                                                 parsedGrammar.c_str(),    // base grammar
                                                 parsedGrammarText.c_str() // additional grammar
                                               );

      // Check for pending TT digit in the digit queue
      if((status = pInfo->ivBoard->QueryDigit(iNext)) != IVR_SUCCESS)
      {
         return(status);
      }

      if(iNext != '\0')
      {

         if((status = pInfo->ivBoard->GetDigit(iNext, iTimeout)) != IVR_SUCCESS)
         {
            return(status);
         }

         // GOT A TOUCH-TONE!
         ttDig = iNext;

         // Touch tones have confidence 100%
         confidence = 100;

         if(!confidenceVar.empty())
         {
            // Set confidence variable with confidence
            // Touch tones get 100% confidence automatically
         	if(!pInfo->variables->Set(confidenceVar.c_str(), "100"))
            {
               pInfo->ivLog->Print("      ERROR: Failed to set Confidence variable '%s' to '100'\n", confidenceVar.c_str());
               return(IVR_FATAL);
            }
         }
      }
      // Start SR
      else
      {
         pInfo->speechRecognition->SRSetAudioFormat(pInfo->ivBoard->GetSRAudioFormat());

         if(!parsedGrammar.empty())
         {
            if((status = pInfo->speechRecognition->SRLoadGrammar(Voice::FILE, parsedGrammar.c_str())) != IVR_SUCCESS)
            {
               pInfo->ivLog->Print("      ERROR: Failed to load speech recognition grammar '%s'\n", parsedGrammar.c_str());
               return(status);
            }
            else if(!parsedGrammarText.empty()
               && ((status = pInfo->speechRecognition->SRAddGrammar(Voice::TEXT, parsedGrammarText.c_str())) != IVR_SUCCESS))
            {
               pInfo->ivLog->Print("      ERROR: Failed to load speech recognition grammar '%s'\n", parsedGrammarText.c_str());
               return(status);
            }
         }
         else if((status = pInfo->speechRecognition->SRLoadGrammar(Voice::TEXT, parsedGrammarText.c_str())) != IVR_SUCCESS)
         {
            pInfo->ivLog->Print("      ERROR: Failed to load speech recognition grammar '%s'\n", parsedGrammarText.c_str());
            return(status);
         }

         // Recognise Audio
       	status = pInfo->ivBoard->AudioRecognise(iTimeout, endOfSpeechTimeout);

         if(status != IVR_SUCCESS)
         {
            pInfo->speechRecognition->SREnd();
            pInfo->speechRecognition->SRClearAudio();
         }

         // No voice detected
      	if(status == IVR_NO_VOICE)
         {
            // Check for TT, will return IVR_TIMEOUT if no digit
            if(((status = pInfo->ivBoard->GetDigit(iNext, 0)) != IVR_SUCCESS) && (status != IVR_TIMEOUT))
            {
               // If hangup detected, that is what should be returned, otherwise return fatal
               return(status == IVR_HANGUP ? IVR_HANGUP : IVR_FATAL);
            }

            // Touch tones have confidence 100%, and no voice has confidence 0%
            confidence = iNext != '\0' ? 100 : 0;
            ttDig = iNext;

            if(!confidenceVar.empty())
            {
               // Touch tones have confidence 100%, and no voice has confidence 0%
            	sprintf(confidenceString, "%d", confidence);

   	         // Set confidence variable with confidence
   	         if(!pInfo->variables->Set(confidenceVar.c_str(), confidenceString))
            	{
            		pInfo->ivLog->Print("      ERROR: Failed to set Confidence variable '%s' to '%s'\n", confidenceVar.c_str(), confidenceString);
   			      return(IVR_FATAL);
               }
            }

            // No digit
            if(status == IVR_TIMEOUT)
            {
               // SR Result = NO VOICE
               // Capture SR result for SpeechTrainer
               pInfo->speechRecognition->SRCaptureResult('\0',         // no digit
                                                     "",           // no concept text
                                                     0,            // confidence undefined
                                                     IVR_TIMEOUT); // result

             	if(!handleVar.empty()
                  && !pInfo->variables->Set(handleVar.c_str(), pInfo->speechRecognition->SRGetRecognitionHandle()))
               {
                  pInfo->ivLog->Print("      ERROR: Failed to set handle variable '%s'\n", handleVar.c_str());
            		return(IVR_FATAL);
               }

               return(IVR_TIMEOUT);
            }
         }
         else if(status != IVR_SUCCESS)
         {
            return(status);
         }
         else
         {
            // Get SR Result
            if(pInfo->speechRecognition->SRGetResult(concept, maxlength, confidence) != IVR_SUCCESS)
            {
   	         pInfo->ivLog->Print("      ERROR: Failed to get speech recognition result\n");
               // SR Result = SR FAILURE
               // Capture SR result for SpeechTrainer
               pInfo->speechRecognition->SRCaptureResult('\0',           // no digit
                                                     "",             // no concept text
                                                     0,              // confidence undefined
                                                     IVR_FAIL);      // result

             	if(!handleVar.empty()
                  && !pInfo->variables->Set(handleVar.c_str(), pInfo->speechRecognition->SRGetRecognitionHandle()))
               {
                  pInfo->ivLog->Print("      ERROR: Failed to set handle variable '%s'\n", handleVar.c_str());
            		return(IVR_FATAL);
               }

               pInfo->speechRecognition->SRClearAudio();

               return(IVR_FAIL);
            }

            if((strlen(concept) > 1) ||
             (!isdigit(concept[0]) && (concept[0] != '*') && (concept[0] != '#')))
            {
               nonMenuSRValue = true;
            }

            iNext = concept[0];

         	if(!confidenceVar.empty())
            {
   	         // Write confidence to confidence string
      	      sprintf(confidenceString, "%d", confidence);

   	         // Set confidence variable with confidence
         	   if(!pInfo->variables->Set(confidenceVar.c_str(), confidenceString))
            	{
         	   	pInfo->ivLog->Print("      ERROR: Failed to set Confidence variable '%s' to '%s'\n", confidenceVar.c_str(), confidenceString);
                  pInfo->speechRecognition->SREnd();
                  pInfo->speechRecognition->SRClearAudio();
   	   		   return(IVR_FATAL);
            	}
            }
         }

         // Check threshold confidence
	      if(!thresholdConfidence.empty())
         {
            // Recognition confidence is less than the minimum or empty string
            if((confidence < iThresholdConfidence) || (iNext == '\0'))
            {
               // SR Result = THRESHOLD FAILURE
               // Capture SR result for SpeechTrainer
               pInfo->speechRecognition->SRCaptureResult('\0',           // no digit
                                                     concept,        // concept text
                                                     confidence,     // confidence
                                                     IVR_THRESHOLD); // result

             	if(!handleVar.empty()
                  && !pInfo->variables->Set(handleVar.c_str(), pInfo->speechRecognition->SRGetRecognitionHandle()))
               {
                  pInfo->ivLog->Print("      ERROR: Failed to set handle variable '%s'\n", handleVar.c_str());
            		return(IVR_FATAL);
               }

               if(!retainAudioOnThreshold)
               {
                  pInfo->speechRecognition->SRClearAudio();
               }

         	   return(IVR_THRESHOLD);
            }
         }
      }
      // endif(iNext != '\0')
   }
   // endif(pInfo->speechRecognition == NULL)

   if(nonMenuSRValue)
   {
      pInfo->ivLog->Print("      Got: '%s'\n", concept);
      SetNextStep(GetTTOther());

      if((status == IVR_SUCCESS) && !varNm.empty() && !pInfo->variables->Set(varNm.c_str(), concept))
      {
         pInfo->ivLog->Print("      ERROR: Failed to set result variable '%s'\n", varNm.c_str());
         return(IVR_FATAL);
      }
   }
   else
   {
      pInfo->ivLog->Print("      Got: '%c'\n", iNext);

      switch(iNext)
      {
         case '0': SetNextStep(GetTT(0)); break;
         case '1': SetNextStep(GetTT(1)); break;
         case '2': SetNextStep(GetTT(2)); break;
         case '3': SetNextStep(GetTT(3)); break;
         case '4': SetNextStep(GetTT(4)); break;
         case '5': SetNextStep(GetTT(5)); break;
         case '6': SetNextStep(GetTT(6)); break;
         case '7': SetNextStep(GetTT(7)); break;
         case '8': SetNextStep(GetTT(8)); break;
         case '9': SetNextStep(GetTT(9)); break;
         case 'A': SetNextStep(GetTT(10)); break;
         case 'B': SetNextStep(GetTT(11)); break;
         case 'C': SetNextStep(GetTT(12)); break;
         case 'D': SetNextStep(GetTT(13)); break;
         case '*': SetNextStep(GetTT(14)); break;
         case '#': SetNextStep(GetTT(15)); break;
         default:  SetNextStep(0);
      }

      if((status == IVR_SUCCESS) && !pInfo->variables->IsEmpty(varNm.c_str()))
      {
         keyPressed[0] = iNext;
         keyPressed[1] = 0;
         if(!pInfo->variables->Set(varNm.c_str(), keyPressed))
         {
            pInfo->ivLog->Print("      ERROR: Failed to set result variable '%s'\n", varNm.c_str());
            return(IVR_FATAL);
         }
      }
   }

   // SR Result = THRESHOLD PASSED
   if(pInfo->speechRecognition != NULL)
   {
      // Capture SR result for SpeechTrainer
      pInfo->speechRecognition->SRCaptureResult(ttDig,      // digit (may be none)
                                            concept,        // concept text
                                            confidence,     // confidence
                                            IVR_SUCCESS);   // result

      if(!pInfo->variables->IsEmpty(handleVar.c_str())
         && !pInfo->variables->Set(handleVar.c_str(), pInfo->speechRecognition->SRGetRecognitionHandle()))
      {
         pInfo->ivLog->Print("      ERROR: Failed to set handle variable '%s'\n", handleVar.c_str());
         return(IVR_FATAL);
      }

      pInfo->speechRecognition->SRClearAudio();
   }

   return(status);
}
/*StepGetDigit::Execute*/
