#include <stdlib.h>             // Needed for "atoi".
#include <string.h>
#include <ctype.h>
#include "step_parse.h"
#include "../escutil/iv_str.h"

StepParse::StepParse(STEPREC& stepRecord) : Step(stepRecord)
{
   parse = stepRecord.s1.arr;
   startPos = stepRecord.s2.arr;
   occurrence = stepRecord.s3.arr;
   varOffset = stepRecord.s4.arr;
   search = stepRecord.s6.arr;
   iSearchType = (PARSE_TYPE) stepRecord.u1;
   iStartCode = (PARSE_START) stepRecord.u2;
   bCaseSensitive = (bool) stepRecord.u3;
   bForward = stepRecord.u4 == 1;
}

StepParse::~StepParse(void)
{
}

const char* StepParse::GetStepName(void) const
{
	return("Parse");
}

// -------------------------------------------------------------------------
//   Name:        StepParse::Execute
//
//   Purpose:     Search a string for a sub-string, single character in a
//                list of characters or a real number.
//
//   Modified:    ??        - JR: Created
//                Aug.28/02 - TF: Revised comments
//                                Added support for "any character of" search
//
//   Notes:       none.
// -------------------------------------------------------------------------
IV_STATUS StepParse::Execute(PIVRINFO pInfo)
{
   string mainStr;
   // Evaluate all parameters before exiting
   bool bParmsValid = true;
   char* value;
   char*     result;
   string parsedSearch;
   int       iStartPos;
   int       iParsedLen;
   int       iOccurrence;
   int       iOffset;

   if(Status(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FAIL);
   }

   if((iSearchType == PT_TEXT) || (iSearchType == PT_ANYCHAR))
   {
      if((value = pInfo->variables->ParseParameter("Desired text/char", search.c_str())) == NULL)
      {
         bParmsValid = false;
      }
      else
      {
         parsedSearch = value;
         delete[] value;
      }
   }
   else if(iSearchType == PT_ELEMENT)
   {
      if((value = pInfo->variables->ParseParameter("Delimiter", search.c_str())) == NULL)
      {
         bParmsValid = false;
      }
      else
      {
         parsedSearch = value;
         delete[] value;
      }
   }

   if((value = pInfo->variables->ParseParameter("Occurence #", occurrence.c_str(), "1")) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      iOccurrence = atoi(value);
      delete[] value;

    	if(iOccurrence < 1)
      {
         pInfo->ivLog->Print("      ERROR: Occurrence '%s' must be at least 1\n", occurrence.c_str());
         bParmsValid = false;
      }
   }

   if((value = pInfo->variables->ParseParameter("Parse string", parse.c_str(), "")) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      mainStr = value;
      delete[] value;
      iParsedLen = mainStr.length();
   }

   switch(iStartCode)
   {
      case PS_BEGIN:
      {
         pInfo->ivLog->Print("         Starting at beginning of text.\n");
         iStartPos = 0;
         break;
      }

      case PS_END:
      {
         pInfo->ivLog->Print("         Starting at end of text.\n");
         iStartPos = iParsedLen - 1;
         break;
      }

      case PS_CURRENT:
      {
         pInfo->ivLog->Print("      ERROR: 'Current Position' no longer supported.  Use 'Defined Position' instead.\n");
         return(IVR_NOTSUPPORTED);
      }

      case PS_SPECIFIC:
      {
         if((value = pInfo->variables->ParseParameter("Defined Position", startPos.c_str(), "1")) == NULL)
         {
            bParmsValid = false;
         }
         else
         {
            iStartPos = atoi(value);
            delete[] value;
            if(iStartPos > 0)
            {
                iStartPos--;     // 0 based index
            }

            if((iStartPos > iParsedLen) || (iStartPos < 0))
            {
               pInfo->ivLog->Print("      ERROR: Parse received invalid starting position:"
                            " %d\n", iStartPos + 1);
               bParmsValid = false;
            }
            else
            {
               pInfo->ivLog->Print("         Starting at %i.\n", iStartPos + 1);     // +1 for human one-based numbering
            }
         }

         break;
      }

      default:
      {
         pInfo->ivLog->Print("      ERROR: Invalid 'Search from' code, %i.\n", iStartCode);
         bParmsValid = false;
      }
   }

   if(!bParmsValid)
   {
      return(IVR_FATAL);
   }

   switch(iSearchType)
   {
      case PT_TEXT:
      {
         pInfo->ivLog->Print("      Parsing %s for '%s'\n", (bForward ? "forward" : "backwards"), parsedSearch.c_str());
         iOffset = FindString(mainStr.c_str(), iStartPos, iOccurrence, parsedSearch.c_str(), bForward, bCaseSensitive);
         break;
      }

      case PT_NUMBER:
      {
         pInfo->ivLog->Print("      Parsing %s for next real number\n", (bForward ? "forward" : "backwards"));
         iOffset = FindNumber(mainStr.c_str(), iStartPos, iOccurrence, bForward);
         break;
      }

      case PT_ANYCHAR:
      {
         pInfo->ivLog->Print("      Parsing %s for any character of '%s'\n",
                      (bForward ? "forward" : "backwards"),
                      parsedSearch.c_str());
         iOffset = FindAnyChar(mainStr.c_str(), iStartPos, iOccurrence, parsedSearch.c_str(), bForward, bCaseSensitive);
         break;
      }

      case PT_ELEMENT:
      {
         const char* suffix = "th";
         if((iOccurrence % 10 == 1) && (iOccurrence % 100 != 11))
         {
            suffix = "st";
         }
         else if((iOccurrence % 10 == 2) && (iOccurrence % 100 != 12))
         {
            suffix = "nd";
         }
         else if((iOccurrence % 10 == 3) && (iOccurrence % 100 != 13))
         {
            suffix = "rd";
         }

         pInfo->ivLog->Print("      Parsing %s for %d%s element delimited by case %ssensitive '%s'\n",
                      (bForward ? "forward" : "backwards"), iOccurrence, suffix,
                      (bCaseSensitive ? "" : "in"), parsedSearch.c_str());
         result = FindDelimitedElement(mainStr.c_str(), iStartPos, iOccurrence, parsedSearch.c_str(), bForward, bCaseSensitive);
         break;
      }
      default:
      {
         pInfo->ivLog->Print("      ERROR: Invalid 'Search for' code, %i\n", iSearchType);
      	return(IVR_FATAL);
      }
   }

   if(iSearchType == PT_ELEMENT)
   {
       if(result == NULL)
       {
         return(IVR_FAIL);
      }
   }
   else if(iOffset < 0)
   {
      return(IVR_FAIL);
   }
   else
   {
      result = new char[8];
      sprintf(result, "%i", iOffset + 1);
   }

   if(!varOffset.empty() && !pInfo->variables->Set(varOffset.c_str(), result))
   {
      pInfo->ivLog->Print("      ERROR: Failed to set variable: '%s'\n", varOffset.c_str());
      delete[] result;
      return(IVR_FATAL);
   }

   delete[] result;
   
   return(IVR_SUCCESS);
}
// StepParse::Execute


int StepParse::FindString (
   const char *  pszText,
   int           iStartPos,
   int           iOccurrence,
   const char *  pszSearch,
   bool          bForward,
   bool          bCaseSensitive)
{
   int    iTextLen;
   int    iSearchLen;
   bool   bFound;
   int    iFoundCount;
   int    iOffset;
   int    iCmpReslt;

   if (!iOccurrence)
      return -3;

   iTextLen = strlen (pszText);
   iSearchLen = strlen (pszSearch);
   iOffset = iStartPos;
   iFoundCount = 0;

   if (iStartPos < 0 || iStartPos > iTextLen)
      return -2;

   bFound = false;
   while (!bFound && iOffset >= 0 && iOffset <= iTextLen) {
      if (bCaseSensitive)
         iCmpReslt = strncmp (pszText + iOffset, pszSearch, iSearchLen);
      else
         iCmpReslt = strnicmp (pszText + iOffset, pszSearch, iSearchLen);

      if (!iCmpReslt) {
         ++iFoundCount;
         if (iFoundCount >= iOccurrence)
            bFound = true;
         else
            bForward ? ++iOffset : --iOffset;
      } else
         bForward ? ++iOffset : --iOffset;
   }

   if (!bFound)
      return -1;

   return iOffset;
}
// StepParse::FindString

// -------------------------------------------------------------------------
//   Name:        StepParse::FindNumber
//
//   Purpose:     Return the offset into the string where the specified
//                occurance of a number starts.
//
//   Notes:       - The following table shows valid numbers.
//
//                Number        Valid? Comments
//                ------------- ---    ---------------------------------------
//                     0        yes
//                  1234        yes
//                 -1234        yes
//                 +1234        yes
//                  1234.       yes    the number: 1234
//                  1234.0      yes
//                  1234.5678   yes
//                     0.5678   yes
//                      .5678   no
//                     -.5678   no
//                 1,234        no     two numbers: 1 and 234
//                 1,234.5678   no     two numbers  1 and 234.5678
//                 (1234)       yes    number 1234 *not* -1234
//                 (1234.5678)  yes
// -------------------------------------------------------------------------
int StepParse::FindNumber (
   const char *  pszText,
   int           iStartPos,
   int           iOccurrence,
   bool          bForward)
{
   int    iTextLen;
   bool   bFound;
   bool   bPoint;
   int    iFoundCount;
   int    iOffset;

   if (!iOccurrence)
      return -3;

   iTextLen = strlen (pszText);
   iOffset = iStartPos;
   iFoundCount = 0;

   if (iStartPos < 0 || iStartPos > iTextLen)
      return -2;

   if (!bForward) {                         // Searching backwards for numbers.
      bFound = false;

      // Search for the nth number.  If found, move to start of number.
      while (!bFound && iOffset >= 0) {
         bPoint = false;
         if (isdigit (pszText[iOffset])) {
            ++iFoundCount;

            // Move to the beginning of the number.
            while (!bFound && iOffset >= 0) {
               if (pszText[iOffset] == '.') {
                  if (bPoint) {
                     bFound = true;
                     ++iOffset;                // Gone too far.  Move back one.
                  } else {
                     bPoint = true;
                     --iOffset;
                  }
               } else if (pszText[iOffset] == '+')
                  bFound = true;
               else if (pszText[iOffset] == '-')
                  bFound = true;
               else if (isdigit (pszText[iOffset])) {
                  --iOffset;
               } else {
                  bFound = true;
                  ++iOffset;                   // Gone too far.  Move back one.
               }
            }
         } else
            --iOffset;

         if (bFound && iFoundCount < iOccurrence) {
            bFound = false;
            --iOffset;
         }
      }

   // If we are searching forward, stop when we find a digit, '+', '-' or '.'.
   } else {
      bFound = false;
      while (!bFound && iOffset <iTextLen) {
         if (pszText[iOffset] == '-' || pszText[iOffset] == '+') {
            if (isdigit (pszText[iOffset + 1])) {
               bFound = true;
               ++iFoundCount;
            } else
               ++iOffset;
         } else if (isdigit (pszText[iOffset])) {
            bFound = true;
            ++iFoundCount;
         } else
            ++iOffset;

         if (bFound && iFoundCount < iOccurrence) {
            bPoint = false;

            if (pszText[iOffset] == '+' || pszText[iOffset] == '-' )
               ++iOffset;

            while (bFound && pszText[iOffset]) {
               if (pszText[iOffset] == '.') {
                  if (bPoint)
                     bFound = false;
                  else {
                     bPoint = true;
                     ++iOffset;
                  }

               } else if (isdigit (pszText[iOffset]))
                  ++iOffset;

               else
                  bFound = false;
            }
            bFound = false;       // Ensure variable is false even in case EOS.
         }
      }
   }

   if (!bFound)
      return -1;

   return iOffset;
}
// FindNumber

int StepParse::FindAnyChar (
   const char *  pszText,
   int           iStartPos,
   int           iOccurrence,
   const char *  pszSearch,
   bool          bForward,
   bool          bCaseSensitive)
{
   int    iTextLen;
   bool   bFound;
   int    iFoundCount;
   int    iOffset;
   const char * iCmpReslt;


   if (!iOccurrence)
      return -3;

   iTextLen = strlen (pszText);
   iOffset = iStartPos;
   iFoundCount = 0;

   if (iStartPos < 0 || iStartPos > iTextLen)
      return -2;

   bFound = false;
   while (!bFound && iOffset >= 0 && iOffset < iTextLen)
   {
      if (bCaseSensitive)
      {
         iCmpReslt = strchr(pszSearch, pszText[iOffset]);
      }
      else
      {
         iCmpReslt = strchr( strlwr((char *)pszSearch), tolower(pszText[iOffset]) );
      }
      if (iCmpReslt)
      {
         ++iFoundCount;
         if (iFoundCount >= iOccurrence)
            bFound = true;
         else
            bForward ? ++iOffset : --iOffset;
      }
      else
      {
         bForward ? ++iOffset : --iOffset;
      }
   }

   if (!bFound)
      return -1;

   return iOffset;
}
// StepParse::FindAnyChar
