// -------------------------------------------------------------------------
// File:        LoadLicense.cpp
//
// Purpose:     These functions are used to map and unmap functions in the
//              TF License DLL into the address of the current process by
//              dynamically loading the DLL at runtime.
//
// Notes:       - This file only needs to be modified if a new function will
//                be exported from ALL board DLLs.
//   -------------------------------------------------------------------------

#include <exception>
using std::exception;

#include "..\tfserver\license_read.h"
#include "LoadLicense.h"

// Class implementation

LICENSE_INFO::LICENSE_INFO(void)
{
   hLibrary = NULL;
   dllGetLicense = NULL;
   licenseData.licensetype = UNLICENSED;
   licenseData.numports = 0;
   licenseData.numdays = 0;
   LoadLicenseModule();
}

LICENSE_INFO::~LICENSE_INFO(void)
{
   if(hLibrary != NULL)
   {
      FreeLibrary(hLibrary);
   }
}

// -------------------------------------------------------------------------
// Name:        LoadLicenseModule
//
// Purpose:     Loads the TF license module DLL, maps to its functions,
//              and populates our local license data structure
// -------------------------------------------------------------------------
bool LICENSE_INFO::LoadLicenseModule(void)
{
   if(hLibrary == NULL)
   {
      // Load license module DLL, or increment count if already loaded
      hLibrary = LoadLibrary("tflicense.dll");

      if(hLibrary == NULL)
      {
         // Failed to load license DLL
         return(false);
      }
   }

   // Load creation and destruction functions from language module
   if(LoadLicenseFunctions())
   {
      if(GetLicenseData())
      {
         return(true);
      }
   }
   return(false);
}


// -------------------------------------------------------------------------
// Name:        LoadLicenseFunctions
//
// Purpose:     Sets reference(s) to the function(s) in TF license DLL
// -------------------------------------------------------------------------
bool LICENSE_INFO::LoadLicenseFunctions(void)
{
#ifdef _MSC_VER
   const LPCSTR getLicenseInfo = "GetLicenseInfo2010";
#else
   const LPCSTR getLicenseInfo = "_GetLicenseInfo2010";
#endif

   if(dllGetLicense == NULL)
   {
      dllGetLicense = (lpFuncGetLicense) GetProcAddress(hLibrary, getLicenseInfo);
   }

   return(dllGetLicense != NULL);
}


// -------------------------------------------------------------------------
// Name:        GetLicenseData
//
// Purpose:     Retrieves license information via TF license DLL
// -------------------------------------------------------------------------
bool LICENSE_INFO::GetLicenseData()
{
   if(dllGetLicense == NULL)
   {
      return(false);
   }

   try
   {
      licenseData = dllGetLicense();
      return(true);
   }
   catch(const exception& error)
   {
      licenseData.licensetype = UNLICENSED;
      licenseData.numports = 0;
      licenseData.numdays= 0;
      return(false);
   }
}

// -------------------------------------------------------------------------
// Name:        GetLicenseStatus
//
// Purpose:     Retrieve the license status
// -------------------------------------------------------------------------
const LICENSE_STATUS LICENSE_INFO::GetLicenseStatus() const
{
   return(licenseData.licensetype);
}

// -------------------------------------------------------------------------
// Name:        GetNumPorts
//
// Purpose:     Retrieve the number (count) of ports licensed
// -------------------------------------------------------------------------
const int LICENSE_INFO::GetNumPorts() const
{
   return(licenseData.numports);
}

// -------------------------------------------------------------------------
// Name:        GetNumDays
//
// Purpose:     Retrieve the number of days left in a temporary license
// -------------------------------------------------------------------------
const int LICENSE_INFO::GetNumDays() const
{
   return(licenseData.numdays);
}
