#include "..\escutil\ESCTime.h"
#include "..\escutil\iv_str.h"
#include "license_read.h"    // TeleFlow license information
#include "TCPClientManager.h"

unsigned int __stdcall TCPClientManagerMain(void* clientManager)
{
   ((TCPClientManager*) clientManager)->Run();
   return(0);
}

TCPClientManager::TCPClientManager(unsigned short binport,
                                   unsigned short httpport,
                                   CommunicationHub<TFServerCommunicationPacket::TF_DESTINATION_ID>& commHub,
                                   TFServerRegistrationInfo* pRegInfo)
   :  connectionRequestBinSocket(monLog, binport),
      connectionRequestHTTPSocket(monLog, httpport),
      commHub(commHub),
      waitObjects(3),
      binaryInterfacePort(binport),
      httpInterfacePort(httpport)
{
   // Capture registry info
   logfolder = pRegInfo->pszLogDir;
   tfstart = pRegInfo->pszTFStart;

   monLog = new GUILog(pRegInfo->dwMaxLogFileSize, pRegInfo->dwLogLines);

   // Open log
   monLog->Open((logfolder + (string) "\\TFMonitor").c_str(), true, true);
   monLog->Print("Created Monitor TCP client manager\n");

   // Set wait objects array with possible signal events
   waitObjects[0] = GetMessageSignal();
   waitObjects[1] = connectionRequestBinSocket.GetEventHandle();
   waitObjects[2] = connectionRequestHTTPSocket.GetEventHandle();
   monLog->Print("   listening for binary communications on port %i...\n", binport);
   monLog->Print("   listening for HTTP communications on port %i...\n", httpport);
}

TCPClientManager::~TCPClientManager(void)
{
   monLog->Print("Monitor TCP client manager closed\n");
   monLog->Close();
   delete monLog;
   Clear();
}

void TCPClientManager::BroadcastAppMessage(TFEngineCommunicationPacket::TF_ENGINE_EVENT eventID)
{
   // Send a message to TFEngine with TFEngineCommunicationPack with 'appID' set to 0
   // means send to all TFEngine threads
   monLog->Print("   Broadcast message TF_ENGINE_EVENT %i to all applications\n", eventID);
   commHub.SendMessage(TFServerCommunicationPacket::TF_ENGINE_MANAGER, new TFEngineCommunicationPacket(0, eventID));
}

void TCPClientManager::Clear(void)
{
   // DO NOT call monLog methods, such as Print(), in here;
   // monLog has likely been deleted by this point.

   for(vector<TCPSocket*>::iterator client = clients.begin(); client != clients.end(); client++)
   {
      delete *client;
   }

   clients.clear();

   waitObjects.erase(waitObjects.begin() + 3, waitObjects.end());
}

TFServerCommunicationPacket::TF_DESTINATION_ID TCPClientManager::ID(void)
{
   return(TFServerCommunicationPacket::TF_TCPCLIENT_MANAGER);
}

void TCPClientManager::ProcessCurrentClientRequest(void)
{
   const char* message;
   int messageLength = sizeof(NET_MSG);
   int bytesWaiting, bytesToReceive;
   ACE_INET_Addr addrLocal;
   ACE_INET_Addr addrRemote;
   int hostPort;
   int result;
   size_t pos = 0;

   if((result = currentClient->GetBytesWaiting(bytesWaiting)) != Socket::SUCCESS)
   {
      monLog->Print("   Failed to retrieve number of bytes waiting, '%s'\n",
                   Socket::WSAErrorString(result));
      RemoveCurrentClient();
      return;
   }

   // Need to know port the message came in on
   // determines if we should interpret the request as HTTP or binary
   result = currentClient->get_local_addr(addrLocal);
   if(result == Socket::SUCCESS)
   {
      hostPort = addrLocal.get_port_number();
      currentClient->get_remote_addr(addrRemote);
   }

   if(hostPort == httpInterfacePort)
   {
      bytesToReceive = bytesWaiting;
   }
   else if (hostPort == binaryInterfacePort)
   {
      bytesToReceive = messageLength;
   }
   else
   {
      monLog->Print("   No interface avaliable on port %i\n", hostPort);
      RemoveCurrentClient();
      return;
   }

   if(currentClient->ReceiveBytes(message, bytesToReceive) != Socket::SUCCESS)
   {
      monLog->Print("   Failed to receive message\n");
      RemoveCurrentClient();
      return;
   }
   // Note: do not delete 'message' buffer!

   // Above method will update actual number of bytes received in 2nd parameter
   messageLength = bytesToReceive;

   if(hostPort == httpInterfacePort)
   {
      string strRequest;
      bool bGoodRequest = false;

      // HTTP request

      // *** Remove comments to have each request output in Monitor log (for debugging only)
      // monLog->Print("   Request (%i bytes)\n", messageLength);
      // monLog->Print("   ------------------\n");
      // monLog->Print("%s\n", message);
      // monLog->Print("   ------------------\n");

      strRequest = message;

      // Check for 'Monitor' keyword
      if((pos = FindCaseInsensitiveString(strRequest.substr(pos).c_str(), "Monitor")) != strRequest.npos)
      {
         // Check for 'Request' keyword
         if((pos = FindCaseInsensitiveString(strRequest.substr(pos).c_str(), "Request")) != strRequest.npos)
         {
            // Check for '=' token
            if((pos = strRequest.find('=', pos)) != strRequest.npos)
            {
               // Check for 'SEND_UPDATE' keyword
               if((pos = FindCaseInsensitiveString(strRequest.substr(pos).c_str(), "SEND_UPDATE")) != strRequest.npos)
               {
                  bGoodRequest = true;
                  SendXMLUpdate();
                  RemoveCurrentClient();
               }
            }
         }
      }

      if(!bGoodRequest)
      {
         // UPDATE messages occur continuously; output to log is not desired.
         // Currently SEND_UPDATE is the only request supported.
         // When others are added, the following line of code will have to move.
         monLog->Print("Message received on port %i from client %s, %s:%i\n",
                       hostPort,
                       addrRemote.get_host_name(),
                       addrRemote.get_host_addr(),
                       addrRemote.get_port_number() );
         //
         monLog->Print("   Invalid request\n");
         monLog->Print(strRequest.c_str());
         SendErrorResponse();
         RemoveCurrentClient();
      }
   }
   else  // hostPort == binaryInterfacePort
   {
      // Binary request

      if(messageLength < sizeof(NET_MSG))
      {
         monLog->Print("   Binary message too short; discarded\n");
         return;
      }

      memcpy(&clientRequest, message, sizeof(NET_MSG));

      // UPDATE messages occur continuously, every second or so output to log is not desired
      if(clientRequest.id != NET_EVT_SEND_UPDATE)
      {
         monLog->Print("Message received on port %i from client %s, %s:%i\n",
                       hostPort,
                       addrRemote.get_host_name(),
                       addrRemote.get_host_addr(),
                       addrRemote.get_port_number() );
         monLog->Print("   %s to application ID %i\n",
                       messageTypeString(clientRequest.id),
                       clientRequest.value);
      }

      switch(clientRequest.id)
      {
         case NET_EVT_SEND_UPDATE:        SendUpdate();  break;
         case NET_EVT_SEND_OUTPUT:        SendOutput();  break;
         case NET_EVT_SD_NORMAL:          SendAppMessage(TFEngineCommunicationPacket::SRV_EVT_SD_NORMAL);  break;
         case NET_EVT_SD_ABORT:           SendAppMessage(TFEngineCommunicationPacket::SRV_EVT_SD_ABORT);  break;
         case NET_EVT_RELOAD_NORMAL:      SendAppMessage(TFEngineCommunicationPacket::SRV_EVT_RELOAD_NORMAL);  break;
         case NET_EVT_RELOAD_ABORT:       SendAppMessage(TFEngineCommunicationPacket::SRV_EVT_RELOAD_ABORT);  break;
         case NET_EVT_ALL_SD_NORMAL:      BroadcastAppMessage(TFEngineCommunicationPacket::SRV_EVT_SD_NORMAL);  break;
         case NET_EVT_ALL_SD_ABORT:       BroadcastAppMessage(TFEngineCommunicationPacket::SRV_EVT_SD_ABORT);  break;
         case NET_EVT_ALL_RELOAD_NORMAL:  BroadcastAppMessage(TFEngineCommunicationPacket::SRV_EVT_RELOAD_NORMAL);  break;
         case NET_EVT_ALL_RELOAD_ABORT:   BroadcastAppMessage(TFEngineCommunicationPacket::SRV_EVT_RELOAD_ABORT);  break;
         case NET_EVT_START_STREAM:       SendAppStreamMessage(TFEngineCommunicationPacket::SRV_EVT_START_STREAM);  break;
         case NET_EVT_STOP_STREAM:        SendAppStreamMessage(TFEngineCommunicationPacket::SRV_EVT_STOP_STREAM);  break;
         case NET_EVT_PAUSE_STREAM:       SendAppStreamMessage(TFEngineCommunicationPacket::SRV_EVT_PAUSE_STREAM);  break;

         case NET_EVT_RELOAD_TAL:
         {
            // Shut down threads
            BroadcastAppMessage(TFEngineCommunicationPacket::SRV_EVT_TERMINATE);

      		// Inform Server that we want to reload
            commHub.SendMessage(TFServerCommunicationPacket::TF_SERVER, new TFServerCommunicationPacket(TFServerCommunicationPacket::SRV_EVT_RELOAD_TAL, 0));

            break;
         }
      }
   }
}

void TCPClientManager::RemoveCurrentClient(void)
{
   vector<TCPSocket*>::iterator clientIterator;

   // Find client
   for(clientIterator = clients.begin();
      (clientIterator != clients.end()) && (*clientIterator != currentClient);
      clientIterator++);

   if(clientIterator != clients.end())
   {
      waitObjects.erase(waitObjects.begin() + 3 + (clientIterator - clients.begin()));
      monLog->Print("Client disconnected\n");
      DisplayRemoteInfo(*clientIterator);
      delete *clientIterator;
      clients.erase(clientIterator);
   }
}

void TCPClientManager::Run(void)
{
   DWORD ret;
   bool terminate = false;

   monLog->Print("Waiting for Monitor events...\n");
   while(!terminate && ((ret = WaitForMultipleObjects(waitObjects.size(), &waitObjects[0], false, INFINITE)) != WAIT_FAILED))
   {
      switch(ret)
      {
         // Message
         case WAIT_OBJECT_0:
         {
            TFTCPClientCommunicationPacket* serverMessage = (TFTCPClientCommunicationPacket*) ReceiveMessage();
            monLog->Print("Server message received, TFTCPClientCommunicationPacket event = %i\n", serverMessage->event);

            switch(serverMessage->event)
            {
               case TFTCPClientCommunicationPacket::TCPCLIENT_EVT_SHUTDOWN:
               {
                  terminate = true;
                  break;
               }

               case TFTCPClientCommunicationPacket::TCPCLIENT_EVT_START_STREAM:
               case TFTCPClientCommunicationPacket::TCPCLIENT_EVT_STOP_STREAM:
               case TFTCPClientCommunicationPacket::TCPCLIENT_EVT_PAUSE_STREAM:
               {
                  NET_MSG clientMessage;
                  vector<TCPSocket*>::iterator client;

                  switch(serverMessage->event)
                  {
                     case TFTCPClientCommunicationPacket::TCPCLIENT_EVT_START_STREAM: clientMessage.id = NET_EVT_START_STREAM;  break;
                     case TFTCPClientCommunicationPacket::TCPCLIENT_EVT_STOP_STREAM: clientMessage.id = NET_EVT_STOP_STREAM;  break;
                     case TFTCPClientCommunicationPacket::TCPCLIENT_EVT_PAUSE_STREAM: clientMessage.id = NET_EVT_PAUSE_STREAM;  break;
                     default: clientMessage.id = NET_EVT_STOP_STREAM;
                  }
                  clientMessage.cause = NET_CAUSE_NONE;
                  clientMessage.value = 0;
                  clientMessage.dataSize = 0;
                  clientMessage.data = NULL;

                  for(client = clients.begin(); (client != clients.end()) && (*client != serverMessage->client); client++);

                  if(client != clients.end())
                  {
                     int socketRet;

                     while((socketRet = (*client)->SendBytes((const char*) &clientMessage, sizeof(NET_MSG))) == WSAEWOULDBLOCK)
                     {
                        Sleep(10);
                     }

                     if(socketRet != Socket::SUCCESS)
                     {
                        currentClient = *client;
                        RemoveCurrentClient();
                     }
                  }

                  break;
               }
            }

            delete serverMessage;

            break;
         }

         // Connection request on binary interface port
         case WAIT_OBJECT_0 + 1:
         {
            TCPSocket* newClient = new TCPSocket(monLog);

            monLog->Print("Connection request received on binary Monitor port %i\n",
                         connectionRequestBinSocket.Get_Port());
            if(connectionRequestBinSocket.Accept(*newClient) == Socket::SUCCESS)
            {
               // monLog->Print("   Accepted\n");
               clients.push_back(newClient);
               waitObjects.push_back(newClient->GetEventHandle());
               DisplayRemoteInfo(newClient); // diplay info in log
            }
            else
            {
               monLog->Print("   Accept failed\n");
               delete newClient;
            }

            break;
         }

         // Connection request on alternate port for HTTP interface
         case WAIT_OBJECT_0 + 2:
         {
            TCPSocket* newClient = new TCPSocket(monLog);

            monLog->Print("Connection request on HTTP Monitor port %i\n",
                         connectionRequestHTTPSocket.Get_Port());
            if(connectionRequestHTTPSocket.Accept(*newClient) == Socket::SUCCESS)
            {
               // monLog->Print("   Accepted\n");
               clients.push_back(newClient);
               waitObjects.push_back(newClient->GetEventHandle());
               DisplayRemoteInfo(newClient); // diplay info in log
            }
            else
            {
               monLog->Print("   Accept failed\n");
               delete newClient;
            }

            break;
         }

         // Client request
         default:
         {
            int events;

            currentClient = clients[ret - (WAIT_OBJECT_0 + 3)];

            if(((ret = currentClient->GetEvents(events)) != Socket::SUCCESS) || (events & FD_CLOSE))
            {
               monLog->Print("   Failed to received message from remote client\n");
               RemoveCurrentClient();
            }
            else
            {
               // Client request received from remote peer (such as TFMonitor or Listener)
               ProcessCurrentClientRequest();
            }

            break;
         }
      }
   }

   Clear();

   if(!terminate)
   {
      monLog->Print("Monitor thread abnormal termination\n");
      commHub.SendMessage(TFServerCommunicationPacket::TF_SERVER, new TFServerCommunicationPacket(TFServerCommunicationPacket::SRV_EVT_MON_TERMINATED, 0));
   }
}

void TCPClientManager::SendAppMessage(TFEngineCommunicationPacket::TF_ENGINE_EVENT eventID)
{
   monLog->Print("   Send message TF_ENGINE_EVENT %i to application ID %i\n",
                 eventID, clientRequest.value);
   commHub.SendMessage(TFServerCommunicationPacket::TF_ENGINE_MANAGER, new TFEngineCommunicationPacket(clientRequest.value, eventID));
}

void TCPClientManager::SendAppStreamMessage(TFEngineCommunicationPacket::TF_ENGINE_EVENT eventID)
{
   TFEngineCommunicationPacket* message;
   unsigned short port;

   message = new TFEngineCommunicationPacket(1000 + clientRequest.value, eventID);

   if(currentClient->GetHostAddress(message->destination) != Socket::SUCCESS)
   {
      delete message;
      return;
   }

   port = (unsigned short) (message->destination.get_port_number() + clientRequest.value);

   if(currentClient->GetPeerAddress(message->destination) != Socket::SUCCESS)
   {
      delete message;
      return;
   }

   message->client = currentClient;
   message->destination.set_port_number(port);

   commHub.SendMessage(TFServerCommunicationPacket::TF_ENGINE_MANAGER, message);
}

void TCPClientManager::SendOutput(void)
{
   int appID = clientRequest.value;
   NET_MSG response;
   char* data;

   response.id = NET_EVT_OUTPUT;
   response.cause = NET_CAUSE_NONE;
   response.value = appID;
   response.data = NULL;

   monLog->Print("   Sending log output for application ID %i\n", appID);

   dataMutex.Get();
   // The following method allocates memory for 'data'; we are responsible for freeing this buffer
   response.dataSize = appsInfo[appID].client_msg.ivLog->GetLines(data);
   dataMutex.Release();
   monLog->Print("   Retrieved %i bytes\n", response.dataSize);

   while(currentClient->SendBytes((const char*) &response, sizeof(NET_MSG)) == WSAEWOULDBLOCK)
   {
      Sleep(10);
   }

   if(response.dataSize > 0)
   {
      while(currentClient->SendBytes(data, response.dataSize) == WSAEWOULDBLOCK)
      {
         Sleep(10);
      }
   }

   delete[] data;
}


void TCPClientManager::SendUpdate(void)
{
   // Send binary update of all application thread activity to TFMonitor
   NET_MSG response;
   CLIENT_MSG* data;
   unsigned int index = 0;

   response.id = NET_EVT_UPDATE;
   response.cause = NET_CAUSE_NONE;
   response.value = 0;
   response.data = NULL;

   dataMutex.Get();
   response.dataSize = appsInfo.size()*sizeof(CLIENT_MSG);
   data = new CLIENT_MSG[appsInfo.size()];
   for(map<unsigned int, EXPANDED_CLIENT_MSG>::iterator app = appsInfo.begin(); app != appsInfo.end(); app++)
   {
      data[index++] = (app->second).client_msg;
   }
   dataMutex.Release();

   while(currentClient->SendBytes((const char*) &response, sizeof(NET_MSG)) == WSAEWOULDBLOCK)
   {
      Sleep(10);
   }

   while(currentClient->SendBytes((const char*) data, response.dataSize) == WSAEWOULDBLOCK)
   {
      Sleep(10);
   }

   delete[] data;
}

void TCPClientManager::SendXMLUpdate(void)
{
   // Send XML update of all application thread activity to the client
   EXPANDED_CLIENT_MSG data;
   const string CRLF = "\r\n";
   char convertbuf[32];    // don't convert anything too big!
   string XMLResponse;
   string Header, Response;
   ESCTime currentTime;

   currentTime.Update(ESCTime::GMT);

   XMLResponse += "<?xml version=\"1.0\"?>" + CRLF;
   XMLResponse += "<TFMonitor ";
   XMLResponse += "logFolder=\"";
   XMLResponse += logfolder;
   XMLResponse += "\" ";
   XMLResponse += "startTime=\"";   // Date/time TFServer started, format "YYYYMMDDHHMMSS"
   XMLResponse += tfstart;
   XMLResponse += "\">" + CRLF;

   dataMutex.Get();
   for(map<unsigned int, EXPANDED_CLIENT_MSG>::iterator app = appsInfo.begin(); app != appsInfo.end(); app++)
   {
      data = app->second;

      XMLResponse += "<TFApp ";
      sprintf(convertbuf,"%i",data.client_msg.msgNo);
      XMLResponse += "msgNo=\"";
      XMLResponse += convertbuf;
      XMLResponse += "\" ";

      sprintf(convertbuf,"%i",data.client_msg.appID);
      XMLResponse += "appID=\"";
      XMLResponse += convertbuf;
      XMLResponse += "\" ";

      XMLResponse += "appName=\"";
      XMLResponse += data.client_msg.appNm;
      XMLResponse += "\" ";

      XMLResponse += "appFile=\"";
      XMLResponse += data.client_msg.appFile;
      XMLResponse += "\" ";

      sprintf(convertbuf,"%i",data.client_msg.lineNo);
      XMLResponse += "Port=\"";
      XMLResponse += convertbuf;
      XMLResponse += "\" ";

      XMLResponse += "currentTAM=\"";
      XMLResponse += data.client_msg.currentTAMName;
      XMLResponse += "\" ";

      sprintf(convertbuf,"%i",data.client_msg.stepNo);
      XMLResponse += "stepNumber=\"";
      XMLResponse += convertbuf;
      XMLResponse += "\" ";

      XMLResponse += "stepName=\"";
      XMLResponse += data.client_msg.stepName;
      XMLResponse += "\" ";

      XMLResponse += "status=\"";
      switch(data.client_msg.status)
      {
         case lsNormal:      XMLResponse += "Normal";     break;
         case lsWarning:     XMLResponse += "Recovered";  break;
         case lsCrashed:     XMLResponse += "Failed";     break;
         case lsNotRunning:  XMLResponse += "Finished";   break;
         case lsEmpty:
         default:
            ;  // do nothing
      }
      XMLResponse += "\" ";

      XMLResponse += "license=\"";
      switch(data.client_msg.license)
      {

         case TEMPORARY:     XMLResponse += "Temporary";  break;
         case LICENSED:      XMLResponse += "Licensed";   break;
         case GPL:           XMLResponse += "GPL";   break;
         case UNLICENSED:
         default:
            XMLResponse += "Unlicensed";
      }
      XMLResponse += "\" ";

      sprintf(convertbuf,"%i",data.client_msg.restartCnt);
      XMLResponse += "restarts=\"";
      XMLResponse += convertbuf;
      XMLResponse += "\" ";

      sprintf(convertbuf,"%i",data.client_msg.restartMax);
      XMLResponse += "restartMax=\"";
      XMLResponse += convertbuf;
      XMLResponse += "\" ";

      XMLResponse += "callerID=\"";
      XMLResponse += data.callingNumber;
      XMLResponse += "\" ";

      XMLResponse += "calledID=\"";
      XMLResponse += data.calledNumber;
      XMLResponse += "\">" + CRLF;

      XMLResponse += "<SysData>";
      XMLResponse += data.client_msg.stepData;
      XMLResponse += "</SysData>" + CRLF;

      XMLResponse += "</TFApp>" + CRLF;
   }
   dataMutex.Release();
   XMLResponse += "</TFMonitor>" + CRLF;

   // Compose the header
   Header = "HTTP/1.1 200 OK" + CRLF;
   //
   Header += "Date: ";
   Header += currentTime.GetHTTPDateTime();
   Header += CRLF;
   //
   Header += "Server: TeleFlow, ";
   Header += TFVERSION;
   Header += CRLF;
   //
   Header += "Content-language: en" + CRLF;
   Header += "Cache-Control: no-cache" + CRLF;
   //
   // "HTTP/1.1 servers SHOULD send Last-Modified whenever feasible"
   Header += "Last-Modified: ";
   Header += currentTime.GetHTTPDateTime();
   Header += CRLF;
   //
   Header += "Content-Length: ";
   sprintf(convertbuf,"%i", XMLResponse.length());
   Header += convertbuf;
   Header += CRLF;
   Header += "Connection: close" + CRLF;
   Header += "Content-Type: text/XML; charset=UTF-8" + CRLF;
   Header += CRLF;

   Response = Header + XMLResponse;

   // *** Remove comments to have each response output in the Monitor log (for debugging only)
   // monLog->Print("      Response \n");
   // monLog->Print("      ------------------\n");
   // monLog->Print("%s\n", Response.c_str());
   // monLog->Print("      ------------------\n");

   // Send XML response to connected client
   while(currentClient->SendBytes((const char*) Response.c_str(), Response.length()) == WSAEWOULDBLOCK)
   {
      Sleep(10);
   }
}

void TCPClientManager::SendErrorResponse(void)
{
   // Respond to invalid HTTP request with an error message
   const string CRLF = "\r\n";
   string Response;
   string Header;
   ESCTime currentTime;

   currentTime.Update(ESCTime::GMT);

   Header = "HTTP/1.1 400 Bad Request" + CRLF;
   //
   Header += "Date: ";
   Header += currentTime.GetHTTPDateTime();
   Header += CRLF;
   //
   Header += "Server: TeleFlow, ";
   Header += TFVERSION;
   Header += CRLF;
   //
   Header += "Content-language: en" + CRLF;
   Header += "Cache-Control: no-cache" + CRLF;
   Header += "Content-Length: 24" + CRLF;
   Header += "Connection: close" + CRLF;
   Header += "Content-Type: text; charset=UTF-8" + CRLF;
   Header += CRLF;

   Response = Header;
   Response += "400 Error: Bad Request" + CRLF;

   monLog->Print("   Responding with '400 Error: Bad Request'\n");

   // Send response to connected client
   while(currentClient->SendBytes((const char*) Response.c_str(), Response.length()) == WSAEWOULDBLOCK)
   {
      Sleep(10);
   }
}

void TCPClientManager::UpdateAppInfo(EXPANDED_CLIENT_MSG& appInfo)
{
   // Not concerned about race conditions with Monitor data
   // Application threads only write, while Monitor thread only reads
   // Data is non-critical, and is only used for display on the TFMonitor
   //
   // dataMutex.Get();
   appsInfo[appInfo.client_msg.appID] = appInfo;
   // dataMutex.Release();
}

void TCPClientManager::UpdateAppStatus(int appID, char status)
{
   // Not concerned about race conditions with Monitor data
   // Application threads only write, while Monitor thread only reads
   // Data is non-critical, and is only used for display on the TFMonitor
   //
   // dataMutex.Get();
   appsInfo[appID].client_msg.status = status;
   // dataMutex.Release();
}

void TCPClientManager::DisplayRemoteInfo(TCPSocket* tcpClient)
{
   if(tcpClient != NULL)
   {
      int result;
      ACE_INET_Addr Address;

      // diplay remote information in monitor log
      result = tcpClient->get_remote_addr(Address);
      if(result == Socket::SUCCESS)
      {
         monLog->Print("   Remote peer '%s', %s:%i\n",
                      Address.get_host_name(),
                      Address.get_host_addr(),
                      Address.get_port_number());
      }
      else
      {
         monLog->Print("   Failed to retrieve local address info\n");
         monLog->Print("      result %i, '%s'\n", result, Socket::WSAErrorString(result));
      }
   }
}

const char* TCPClientManager::messageTypeString(unsigned long msgID)
{
   // Translate a message event sent to the server from the client
   // into char strings for output in Log
   switch(msgID)
   {
      case NET_EVT_STRING            : return("NET_EVT_STRING");
      case NET_EVT_SEND_UPDATE       : return("NET_EVT_SEND_UPDATE");
      case NET_EVT_SD_NORMAL         : return("NET_EVT_SD_NORMAL");
      case NET_EVT_SD_ABORT          : return("NET_EVT_SD_ABORT");
      case NET_EVT_RELOAD_NORMAL     : return("NET_EVT_RELOAD_NORMAL");
      case NET_EVT_RELOAD_ABORT      : return("NET_EVT_RELOAD_ABORT");
      case NET_EVT_ALL_SD_NORMAL     : return("NET_EVT_ALL_SD_NORMAL");
      case NET_EVT_ALL_SD_ABORT      : return("NET_EVT_ALL_SD_ABORT");
      case NET_EVT_SEND_OUTPUT       : return("NET_EVT_SEND_OUTPUT");
      case NET_EVT_ALL_RELOAD_NORMAL : return("NET_EVT_ALL_RELOAD_NORMAL");
      case NET_EVT_ALL_RELOAD_ABORT  : return("NET_EVT_ALL_RELOAD_ABORT");
      case NET_EVT_RELOAD_TAL        : return("NET_EVT_RELOAD_TAL");
      case NET_EVT_START_STREAM      : return("NET_EVT_START_STREAM");
      case NET_EVT_STOP_STREAM       : return("NET_EVT_STOP_STREAM");
      case NET_EVT_PAUSE_STREAM      : return("NET_EVT_PAUSE_STREAM");
      default : return("UNKNOWN");
   }
}
//messageTypeString

