/* -------------------------------------------------------------------------
   File:        ODBC_DB.H

   Purpose:     Defines the ODBC classes used by the TeleFlow Engine
                threads.

                The classes are derived from the Head and Link classes and
                store results from queries into the VarList global list.

   Notes:       - ODBC functions should be independent from Engine
                  requirements.

   ODBC Notes:  * *** WARNING *** Make sure that the project uses the latest
                  ODBC files by placing the ODBC directories first in the
                  "Include" and "Lib" paths under Project -> Options ->
                  Directories/Conditionals.
                  If you don't do this, the project will use the headers
                  and lib files shipped with Borland which are behind in
                  versions with the one downloaded from Microsoft's website.
                - ODBC drivers are available from Microsoft by downloading
                  their Data Access SDK (DASDK or MSDASDK).  Note that MS
                  Data Access Components (MDAC) must be installed first.
                  The last time I downloaded these files they were under the
                  Universal Data Access (UDA) web page.
                     http://www.microsoft.com/data/doc.htm
                - The files are also available on the MS Visual Studio
                  CDROM.

   Copyright (c) 1998 - 2008 Engenic.  All rights reserved.
   ------------------------------------------------------------------------- */
#ifndef ODBC_DB_H
#define ODBC_DB_H

#include <winsock2.h>
#include <windows.h>
#include <stdio.h>
#include <algorithm>
using std::transform;

#ifndef WIN32
#define WIN32
#endif

#include <sql.h>
#include <sqlext.h>

#include "..\escutil\export_declarations.h"
#include "..\TFServer\ivrinfo.h"
#include "..\escutil\link.h"

/* -------------------------------------------------------------------------
   Structure:   SQLParm

   Purpose:     Stores information needed to bind to a parameter marker in
                a stored procedure call.
   ------------------------------------------------------------------------- */
struct CLASS_DECLARATION SQLParm
{
   SQLParm();
   ~SQLParm();
   char        *pszVarName;
   char        *pszColValue;
   SQLUINTEGER  iColSize;
   SQLSMALLINT  iColPrec;
   SQLSMALLINT  ioType;
   SQLSMALLINT  parmType;
   SQLLEN       sqlnts;
};

#if 0
/* -------------------------------------------------------------------------
   Structure:   SQLCall

   Purpose:     Stores information needed to make a stored procedure call.
   ------------------------------------------------------------------------- */
struct CLASS_DECLARATION SQLCall
{
   SQLCall();
   ~SQLCall();
   char    *pszStatement;
   int      iParmCount;
   int      refNo;
   SQLParm *sqlParms;
};
#endif

class SQLList;
class SQL;

// -------------------------------------------------------------------------
// Class:       ODBCManager
//
// Purpose:     Maintains a list of open ODBC connections
// -------------------------------------------------------------------------
class CLASS_DECLARATION ODBCManager
{
   public:
      ODBCManager(PIVRINFO pIVRInfoStruct);
      ~ODBCManager(void);

      // Locate a connection in the ODBC 'ConnectionsList' map
      // returns a pointer to the SQLList if found, or NULL if not found
      SQLList* FindConnection(const char* connectionHandle);

      // Connect to ODBC data source.
      // Store connection information in the 'ConnectionsList' map, using
      // provided 'connectHandle' parameter as the key.
      // Can pass an empty string, "" (but not NULL) as a connection handle;
      // That is the default handle for legacy support.
      IV_STATUS Connect(const char* connectHandle, const char* connectParms);

      // Disconnect the specified 'connectHandle' from its ODBC data source.
      // Remove connection information in the 'ConnectionsList' map
      // And, delete the SQLList object that it points to.
      // Will simply return IVR_SUCCESS if the handle does not exist.
      // Can pass an empty string, "" (but not NULL) as a connection handle;
      // That is the default handle for legacy support.
      IV_STATUS Disconnect(const char* connectHandle);

      // Disconnect all connections to ODBC data sources
      // Remove all connection information from the 'ConnectionsList' map/
      // And, delete the SQLList object that each points to.
      // Which has the effect of invalidating all connection handles.
      void Reset(void);

   private:
      PIVRINFO pInfo;
      Log* ivLog;
      map<string, SQLList*> ConnectionsList;
};

/* -------------------------------------------------------------------------
   Class:       SQLList

   Purpose:     Defines DB session-level functions.
   ------------------------------------------------------------------------- */
class CLASS_DECLARATION SQLList : public Head
{
   public:

      SQLList(PIVRINFO pInfoStruct);
      virtual ~SQLList(void);

      SQL* FindSQL(const char* refName);

      IV_STATUS Connect(const char* connectString);
      IV_STATUS Disconnect(void);
      IV_STATUS BeginTrans(void);
      IV_STATUS EndTrans(bool commitData);
      SQLHDBC GetDBHandle(void);
      bool ConnectionStringMatch(const char* compareStr);
      bool Okay(void);

      void Reset(void);

      IV_STATUS DetermineTransCapability(void);

   private:

      PIVRINFO     pInfo;
      Log*         ivLog;
      bool         okay;
      SQLHENV      henv;
      SQLHDBC      hdbc;          // Handle to the database.
      bool         transOpen;
      string       connectionParms;

      void SaveConnectionString(const char* connectString);
      void SetChildDBHandle(void);
      void SetChildTransOpen(void);
};

/* -------------------------------------------------------------------------
   Class:       SQL

   Purpose:     Defines DB transaction-level functions.
   ------------------------------------------------------------------------- */
class CLASS_DECLARATION SQL : public Link
{
   public:
      SQL(void);
      ~SQL(void);

      IV_STATUS Lock(void);
      IV_STATUS BeginStmt(PIVRINFO pInfo, const char* szQry);
      IV_STATUS Fetch(PIVRINFO pInfo);
      IV_STATUS EndStmt(void);
      int GetFieldCount(void);
      IV_STATUS GetField (int fieldNo, char *value);
      IV_STATUS SetReferenceName(const char* value);
      const char* GetReferenceName() { return refName.c_str(); }
      void SetDBHandle (SQLHDBC value) { hdbc = value; }
      void SetTransOpen (bool value) { transOpen = value; }
      unsigned GetRowsSelected() { return rowCount; }

      IV_STATUS CallProcedure(PIVRINFO pInfo, char* pszStmt, char* pszParams);

   private:

      string       refName;       // TeleFlow statement reference name (handle)
      SQLHDBC      hdbc;          // Handle to the database.
      SQLHSTMT     hstmt;         // Handle to the SQL statement.
      bool         transOpen;
      unsigned     rowCount;

      IV_STATUS BindParameter(PIVRINFO pInfo, SQLUSMALLINT pos, SQLParm* sqlParm);

      bool ParseParamStrings(
         char     *pszStmt,
         char     *pszParams,
         int      *parmCount,
         SQLParm **sqlParms);

      bool ParseParamString(
         char    *pszValue,
         SQLParm *sqlParm);
};

#endif
