#ifndef __SDPINFO_H__
#define __SDPINFO_H__

#ifdef __DLL__
#define SDPINFOAPI __declspec(dllexport)
#else
#define SDPINFOAPI __declspec(dllimport)
#endif

#include <string>
using std::string;
#include <deque>
using std::deque;

#include "..\TFServer\log.h"

// Class for managing SDP information as per RFC 2327
class SDPINFOAPI SDPInfo
{
   public:

      enum NETWORK_TYPE
      {
         INVALID_NETWORK_TYPE = -1,
         INTERNET = 1
      };

      enum ADDRESS_TYPE
      {
         INVALID_ADDRESS_TYPE = -1,
         IP4 = 1
      };

      enum MEDIA
      {
         INVALID_MEDIA = -1,
         AUDIO = 1
      };

      enum PROTOCOL
      {
         INVALID_PROTOCOL = -1,
         RTP = 1
      };

      enum PAYLOAD
      {
         // Payload Type Definitions (as per RFC 3551) supported by TeleFlow
         INVALID_PAYLOAD = -1,
         G711_U = 0,
         GSM = 3,
         G723 = 4,
         G711_A = 8,
         G729 = 18,
         TELEPHONE_EVENT = 101
      };

      SDPInfo(void);

      void AddHostPayload(SDPInfo::PAYLOAD payload);
      void AddPeerPayload(SDPInfo::PAYLOAD payload);
      void ClearHostPayloads(void);
      void ClearPeerPayloads(void);
      // RFC 2327 recommends NTP (RFC 1305), but may be any 64-bit number less than
      // the maximum value as per RFC 3264.  We use approximately the microseconds
      // since epoch to generate this value.
      const char* GenerateSessionID(void);
      const char* GetHostAddress(void) const;
      SDPInfo::PAYLOAD GetHostPayload(unsigned int index) const;
      unsigned short GetHostPort(void) const;
      const char* GetHostUserName(void) const;
      SDPInfo::MEDIA GetMediaType(void) const;
      const char* GetPeerAddress(void) const;
      SDPInfo::PAYLOAD GetPeerPayload(unsigned int index) const;
      unsigned short GetPeerPort(void) const;
      const char* GetPeerUserName(void) const;
      SDPInfo::PAYLOAD GetPrimaryPayload(void) const;
      SDPInfo::PROTOCOL GetProtocol(void) const;
      const char* GetSessionID(void) const;
      const char* GetSessionName(void) const;
      const char* GetVersion(void) const;
      void Print(Log* ivLog) const;
      void PrioritiseHostPayloads(void);
      void PrioritisePeerPayloads(void);
      void Reset(void);
      void SetHostAddress(const char* address);
      void SetHostPayloads(SDPInfo::PAYLOAD* payloads, int count);
      void SetHostPort(unsigned short port);
      void SetHostUserName(const char* name);
      void SetMediaType(SDPInfo::MEDIA mediaType);
      void SetPeerAddress(const char* address);
      void SetPeerPayloads(SDPInfo::PAYLOAD* payloads, int count);
      void SetPeerPort(unsigned short port);
      void SetPeerUserName(const char* name);
      void SetProtocol(SDPInfo::PROTOCOL protocol);
      void SetProtocol(DWORD protocol);
      void SetSessionID(const char* ID);
      void SetSessionName(const char* name);
      void SetVersion(const char* version);

      // Interpret the data in a buffer as SDP (per RFC 2327), and extract
      // information about the remote peer into SDPInfo's peer properties
      // The 'buffer' parameter must be a NULL terminated C string.
      // The 'bufsize' parameter is optional and can be used to limit the parsing
      // to a sub-string of the right-most 'bufsize' characters.
      // If the data is valid SDP, the properties of this class are updated,
      // peerInfoChanged is set to TRUE, and and this method returns TRUE.
      // If the data is not valid, the properties remain unchanged, and this
      // method returns FALSE.
      bool ParseRemoteInfo(const char* buffer, size_t bufsize = 0);

      // Returns true if peer info has changed
      bool IsPeerInfoChanged(void);

      // Resets the peer info changed flag to FALSE
      void ResetPeerInfoChange(void);

      // Compose SDP message for use in a SIP BODY element
      const char* ComposeMessageBody();

   protected:

      string sessionName;
      string sessionID;
      string hostUserName;
      string peerUserName;
      SDPInfo::MEDIA mediaType;
      string hostAddress;
      string peerAddress;
      unsigned short hostPort;
      unsigned short peerPort;
      SDPInfo::PROTOCOL protocol;
      string version;
      bool prioritiseHostPayloads;
      deque<SDPInfo::PAYLOAD> hostPayloads;
      deque<SDPInfo::PAYLOAD> peerPayloads;
      bool peerInfoChanged;         // set to true if peer data changed by ParseRemoteInfo()
      string messageBody;           // set and returned by composeMessageBody()
      SDPInfo::NETWORK_TYPE networkType;
      SDPInfo::ADDRESS_TYPE addressType;

      void AddPayload(deque<SDPInfo::PAYLOAD>& payloadList, SDPInfo::PAYLOAD payload);
      void SetPayloads(deque<SDPInfo::PAYLOAD>& payloadList, SDPInfo::PAYLOAD* payloads, int count);

   private:

      string networkTypeString;
      string addressTypeString;

      // Convert internal network type to a valid string as per RFC 2327
      const char* TranslateNetworkType(void);

      // Convert internal address type to a valid string as per RFC 2327
      const char* TranslateAddressType(void);

      // Return the encoding name for a specific payload ID, for use in the rtpmap attribute
      const char* TranslateEncodingName(SDPInfo::PAYLOAD payload);

      // returns Nth token in a string of SDP values, or empty string if no token at that position.
      string GetValueToken(string values, unsigned int n);
};

#endif
